from django.contrib import admin
from cilantro.models import Settings
from avocado.fields.admin import FieldAdmin
from avocado.models import Field

class ChartEnabledFieldAdmin(FieldAdmin):
    fieldsets = FieldAdmin.fieldsets[:1] + (
        ('Chart Labels', {
            'classes': ('collapse',),
            'fields': ('chart_title', 'chart_xaxis', 'chart_yaxis'),
        }),
    ) + FieldAdmin.fieldsets[1:]


admin.site.unregister(Field)
admin.site.register(Field, ChartEnabledFieldAdmin)
admin.site.register(Settings)



from django.db import models
from django.contrib.sites.models import Site

class Settings(models.Model):
    site = models.OneToOneField(Site, related_name='cilantro_settings')
    site_logo = models.ImageField(upload_to='cilantro/settings', null=True)
    footer_content = models.TextField(null=True, help_text='Markdown enabled')
    google_analytics = models.CharField(max_length=20, null=True,
        help_text='Just the UA-XXXXXX-XXX code')

    class Meta(object):
        verbose_name_plural = 'settings'

    def __unicode__(self):
        return u'Cilantro Settings'

    @property
    def site_name(self):
        return self.site.name

    @property
    def site_domain(self):
        return self.site.domain



from django.shortcuts import render_to_response, redirect
from django.template import RequestContext

from avocado.models import Report
from avocado.modeltree import trees, DEFAULT_MODELTREE_ALIAS
from serrano.api.resources import ColumnResourceCollection

def login_redirect(request):
    # if the user does not have any saved queries, redirect to the define
    # page instead of the their workspace
    if not Report.objects.filter(user=request.user, session=False).count():
        return redirect('define')
    return redirect('workspace')

def workspace(request):
    return render_to_response('workspace.html',
        context_instance=RequestContext(request))

def define(request):
    return render_to_response('define.html',
        context_instance=RequestContext(request))

def report(request):
    resource = ColumnResourceCollection()
    columns = resource.GET(request)
    # XXX: hack.. page components are not yet dynamic
    model_name_plural = trees[DEFAULT_MODELTREE_ALIAS].root_model._meta.verbose_name_plural.format()
    return render_to_response('report.html', {
        'columns': columns,
        'model_name_plural': model_name_plural,
    }, context_instance=RequestContext(request))



from django import template
from django.forms import widgets

register = template.Library()

@register.inclusion_tag('formfield.html')
def formfield(field):
    widget = field.field.widget

    if isinstance(widget, widgets.Input):
        input_type = 'input'
    elif isinstance(widget, widgets.Textarea):
        input_type = 'textarea'
    elif isinstance(widget, widgets.Select):
        input_type = 'select'
    elif isinstance(widget, widgets.CheckboxInput):
        input_type = 'checkbox'
    elif isinstance(widget, widgets.RadioInput):
        input_type = 'radio'
    else:
        input_type = None

    return {'field': field, 'form': field.form, 'type': input_type}



from django.db import models

class FieldMixin(models.Model):
    "Adds charting components."

    chart_title = models.CharField(max_length=100, null=True, blank=True)
    chart_yaxis = models.CharField(max_length=100, null=True, blank=True)
    chart_xaxis = models.CharField(max_length=100, null=True, blank=True)

    class Meta(object):
        abstract = True



from avocado.criteria.viewset import AbstractViewSet, library

CHART_TYPES = ['string','number','nullboolean','boolean']

@library.register
class DefaultViewSet(AbstractViewSet):
    """Constructs a single view with some sensible interpretation of the fields.

    A few assumptions are made when determining the appropriate view to be
    provided::

        - if the ``concept`` contains a single field, a chart is constructed

        - if all of the ``concept's`` fields are required, no chart is
        constructed, since none of the fields seemingly take precendence over
        each other

        - if *not* all of the ``concept's`` fields are required, a chart is
        constructed for the first field (by the order specified)

    """
    def _get_coords(self, field, **kwargs):
        if field.datatype in ("string", "nullboolean"):
            coords = field.distribution(**kwargs)
        else:
            coords = field.distribution(order_by='count', exclude=(None,), **kwargs)
        return coords

    def _is_chartable(self, field):
        if field.datatype in CHART_TYPES:
            # we don't want free text to be graphed
            if field.datatype == 'string' and not field.enable_choices:
                return False
            return True
        return False

    def default_view(self, concept, cfields, *args, **kwargs):
        chart_enabled = False
        elements = []

        form = {
            'type': 'form',
            'fields': [],
        }

        if len(cfields) == 1 or not all([f.required for f in cfields]):
            cfield_0 = cfields[0]
            choices = cfield_0.field.choices
            # only specify a chart if the field is required and is able
            # to be charted
            if cfield_0.required and self._is_chartable(cfield_0.field) and (choices is None or len(choices) <= 15):
                
                cfield_0 = cfields.pop(0)
                field = cfield_0.field
                
                chart = {
                    'type': 'chart',
                    'data': {
                        'datatype': field.datatype,
                        'title': field.chart_title or field.name,
                        'coords': self._get_coords(field),
                        'xaxis': field.chart_xaxis,
                        'yaxis': field.chart_yaxis,
                        'pk' :  field.pk,
                        'name': cfield_0.get_name(),
                        'optional': not cfield_0.required,
                        'choices': field.choices
                    }
                }
                elements.append(chart)

        for cfield in cfields:
            optional = not cfield.required
            field = cfield.field

            row = {
                'name' : cfield.get_name(),
                'datatype' : field.datatype,
                'choices' : field.choices,
                'pk': field.pk,
                'optional' : optional
            }
            form['fields'].append(row)


        if form['fields']:
            elements.append(form)

        viewset = {
            'type': 'builtin',
            'elements': elements
        }

        return viewset





from django import template
from django.template import Variable
from django.template.loader import get_template

register = template.Library()

@register.tag
def render_paginator(parser, token):
    toks = token.split_contents()
    toks.pop(0) # pop tag name
    for i,t in enumerate(toks):
        toks[i] = t.strip('"').strip("'")
    return RenderPaginator(*toks)


class RenderPaginator(template.Node):
    def __init__(self, template_name, first_last_amount=0,
        before_after_amount=2):

        self.template_name = template_name
        self.first_last_amount = int(first_last_amount)
        self.before_after_amount = int(before_after_amount)

    def render(self, context):
        page_obj = Variable('page_obj').resolve(context)
        paginator = Variable('paginator').resolve(context)
        page_numbers = []

        # only process if the current page number is greater than the number of
        # required links.
        if page_obj.number > (self.first_last_amount + self.before_after_amount + 1):
            # append first pages, e.g. [1, 2] if `first_last_amount' is 2
            for i in range(1, self.first_last_amount + 1):
                page_numbers.append(i)

            page_numbers.append(None)
            # append the leading pages relative to the current page, e.g.
            # [10, 11] if `before_after_amount' is 2 and current page is 12
            for i in range(page_obj.number - self.before_after_amount, page_obj.number):
                page_numbers.append(i)

        else:
            for i in range(1, page_obj.number):
                page_numbers.append(i)

        # if there is still a gap until `paginator.num_pages' is reached, add trailing pages
        if (page_obj.number + self.first_last_amount + self.before_after_amount + 1) < paginator.num_pages:
            for i in range(page_obj.number, page_obj.number + self.before_after_amount + 1):
                page_numbers.append(i)

            page_numbers.append(None)
            for i in range(paginator.num_pages + 1 - self.first_last_amount, paginator.num_pages + 1):
                page_numbers.append(i)

        else:
            for i in range(page_obj.number, paginator.num_pages + 1):
                page_numbers.append(i)

        t = get_template(self.template_name)
        context.update({
            'page_numbers': page_numbers,
        })
        return t.render(context)



from django.conf.urls.defaults import patterns, url
from django.template.loader import add_to_builtins

add_to_builtins('cilantro.templatetags.formutils')

urlpatterns = patterns('cilantro.views',
    url(r'^redirect/$', 'login_redirect', name='login-redirect'),
    url(r'^workspace/$', 'workspace', name='workspace'),
    url(r'^define/$', 'define', name='define'),
    url(r'^report/$', 'report', name='report'),
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding model 'Settings'
        db.create_table('cilantro_settings', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('site', self.gf('django.db.models.fields.related.OneToOneField')(related_name='cilantro_settings', unique=True, to=orm['sites.Site'])),
            ('site_logo', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True)),
            ('footer_content', self.gf('django.db.models.fields.TextField')(null=True)),
            ('google_analytics', self.gf('django.db.models.fields.CharField')(max_length=20, null=True)),
        ))
        db.send_create_signal('cilantro', ['Settings'])


    def backwards(self, orm):

        # Deleting model 'Settings'
        db.delete_table('cilantro_settings')


    models = {
        'cilantro.settings': {
            'Meta': {'object_name': 'Settings'},
            'footer_content': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'google_analytics': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'site': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'cilantro_settings'", 'unique': 'True', 'to': "orm['sites.Site']"}),
            'site_logo': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['cilantro']



from django.contrib.sites.models import Site

def media(request):
    from django.conf import settings
    static_url = getattr(settings, 'CILANTRO_STATIC_URL', None)

    if static_url is None:
        static_url = settings.STATIC_URL + 'cilantro/'

    javascript_url = '{0}scripts/javascript/{1}/'.format(static_url,
        'src' if settings.DEBUG else 'min')

    css_url = '{0}stylesheets/css/'.format(static_url)

    return {
        'CILANTRO_STATIC_URL': static_url,
        'CILANTRO_JAVASCRIPT_URL': javascript_url,
        'CILANTRO_CSS_URL': css_url,
    }

def settings(request):
    site = Site.objects.get_current()
    return {
        'CILANTRO': site.cilantro_settings,
    }
